/*
 * Created on Sep 5, 2005
 *
 */
package com.octo.cas.client.websphere;

import java.net.URLEncoder;
import java.util.Properties;

import javax.servlet.http.HttpServletRequest;

import edu.yale.its.tp.cas.client.ServiceTicketValidator;

/**
 * <a
 * href="http://www-128.ibm.com/developerworks/views/download.jsp?contentid=92125&filename=WASAdvancedAuthentication.zip&method=http&locale=worldwide">
 * Trusted Association Interceptor Helper </a> for CAS. <br/>This TAI Helper
 * provide methods to configure, intercept calls containing the "ticket"
 * parameter (Added by a CAS server after successfull authentication), and check
 * against a CAS server that the ticket is valid for the requested service.
 * <br/>It also uses this ticket to retrieve the real user name information.
 * <br/><br/>See static final properties documentation for configuration.
 * 
 * This helper is used by CasTAI511 and CasTAI502
 * 
 * @author Marc-Antoine Garrigue
 */
public class CasTAIHelper {

	private static final String DEFAULT_PRINCIPAL_PREFIX = "";

	private static final String DEFAULT_PRINCIPAL_SUFFIX = "";

	private static final String DEFAULT_CAS_PROXY_SERVLET = "/CasProxyServlet";

	public static final String DEFAULT_REALM_NAME = "CAS_REALM";

	/**
	 * Constant string representing the ticket parameter.
	 */
	public static final String PARAM_TICKET = "ticket";

	/**
	 * <b>required </b> URL of the CAS server to validate the ticket to
	 */
	public static final String CAS_VALIDATION_URL_PROP = "CAS_VALIDATION_URL";

	/**
	 * <b>required if STORE_PROXY_TICKET_PROP=true </b> Base url for the
	 * callback of the proxy ticket. The resulting url will be
	 * <code> CAS_CALLBACK_PROXY_URL_PROP+request.getContextPath()+"/CasProxySerlvet"</code>.
	 * <br/>Default is null;
	 */
	public static final String CAS_CALLBACK_PROXY_URL_PROP = "CAS_CALLBACK_PROXY_URL";

	/**
	 * <b>required if STORE_PROXY_TICKET_PROP=true </b> Base url for the
	 * callback of the proxy ticket. The resulting url will be
	 * <code> CAS_CALLBACK_PROXY_URL+request.getContextPath()+CAS_CALLBACK_PROXY_SERVLET</code>.
	 * <br/> Default : '/CasProxySerlvet'
	 */
	public static final String CAS_CALLBACK_PROXY_SERVLET_PROP = "CAS_CALLBACK_PROXY_SERVLET";

	/**
	 * <b>optional </b> Debug to System.out? default false.
	 */
	public static final String DEBUG_PROP = "DEBUG";

	/**
	 * <b>optional </b> Debug to System.out? default false.
	 */
	public static final String STORE_PROXY_TICKET_PROP = "STORE_PROXY_TICKET";

	/**
	 * <b>optional </b> Prefix to add to information returned by CAS in the
	 * principal. Default is blanck
	 * 
	 */
	public static final String PRINCIPAL_PREFIX_PROP = "PRINCIPAL_PREFIX";

	/**
	 * <b>optional </b> Suffix to add to information returned by CAS in to the
	 * principal. Default is blanck
	 * 
	 */
	public static final String PRINCIPAL_SUFFIX_PROP = "PRINCIPAL_SUFFIX";

	/**
	 * <b>optional </b> Realm of this TAI. Default "CAS_REALM"
	 */
	public static final String CAS_REALM_NAME_PROP = "CAS_REALM_NAME_PROP";

	protected static String CAS_VALIDATION_URL;

	protected static String CAS_CALLBACK_PROXY_URL;

	protected static String CAS_CALLBACK_PROXY_SERVLET = DEFAULT_CAS_PROXY_SERVLET;

	protected static String CAS_REALM_NAME = DEFAULT_REALM_NAME;

	protected static String PRINCIPAL_PREFIX = DEFAULT_PRINCIPAL_PREFIX;

	protected static String PRINCIPAL_SUFFIX = DEFAULT_PRINCIPAL_SUFFIX;

	protected static boolean DEBUG = false;

	protected static boolean STORE_PROXY_TICKET = false;

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.wsspi.security.tai.TrustAssociationInterceptor#initialize(java.util.Properties)
	 */
	public static int initialize(Properties props) {
		try {
			if (props != null) {
				CasTAIHelper.CAS_VALIDATION_URL = props
						.getProperty(CasTAIHelper.CAS_VALIDATION_URL_PROP); // extract

				if (props.getProperty(CasTAIHelper.CAS_CALLBACK_PROXY_URL_PROP) != null) {
					CasTAIHelper.CAS_CALLBACK_PROXY_URL = props
							.getProperty(CasTAIHelper.CAS_CALLBACK_PROXY_URL_PROP); // extract
				} else {
					CasTAIHelper.CAS_CALLBACK_PROXY_URL = null;
				}

				if (props
						.getProperty(CasTAIHelper.CAS_CALLBACK_PROXY_SERVLET_PROP) != null) {
					CasTAIHelper.CAS_CALLBACK_PROXY_SERVLET = props
							.getProperty(CasTAIHelper.CAS_CALLBACK_PROXY_SERVLET_PROP);
				} else {
					CasTAIHelper.CAS_CALLBACK_PROXY_SERVLET = DEFAULT_CAS_PROXY_SERVLET;
				}

				if (props.getProperty(CasTAIHelper.CAS_REALM_NAME_PROP) != null) {
					CasTAIHelper.CAS_REALM_NAME = props
							.getProperty(CasTAIHelper.CAS_REALM_NAME_PROP);
				} else {
					CasTAIHelper.CAS_REALM_NAME = DEFAULT_REALM_NAME;
				}
				if (props.getProperty(CasTAIHelper.PRINCIPAL_PREFIX_PROP) != null) {
					CasTAIHelper.PRINCIPAL_PREFIX = props
							.getProperty(CasTAIHelper.PRINCIPAL_PREFIX_PROP); // extract
				} else {
					CasTAIHelper.PRINCIPAL_PREFIX = DEFAULT_PRINCIPAL_PREFIX;
				}
				if (props.getProperty(CasTAIHelper.PRINCIPAL_SUFFIX_PROP) != null) {
					CasTAIHelper.PRINCIPAL_SUFFIX = props
							.getProperty(CasTAIHelper.PRINCIPAL_SUFFIX_PROP); // extract
				} else {
					CasTAIHelper.PRINCIPAL_SUFFIX = DEFAULT_PRINCIPAL_SUFFIX;
				}
				if (props.getProperty(CasTAIHelper.DEBUG_PROP) != null) {
					CasTAIHelper.DEBUG = Boolean.valueOf(
							props.getProperty(CasTAIHelper.DEBUG_PROP))
							.booleanValue();

				} else {
					CasTAIHelper.DEBUG = false;
				}
				if (props.getProperty(CasTAIHelper.STORE_PROXY_TICKET_PROP) != null) {
					CasTAIHelper.STORE_PROXY_TICKET = Boolean
							.valueOf(
									props
											.getProperty(CasTAIHelper.STORE_PROXY_TICKET_PROP))
							.booleanValue();

				} else {
					CasTAIHelper.STORE_PROXY_TICKET = false;
				}
			}
		} catch (Exception e) {
			System.err.println("error during CasTAIHelper init : "
					+ e.getMessage());
			return -1;
		}

		System.out.print("\n\n CasTAIHelper configuration : "
				+ "\n\t CAS_REALM_NAME='" + CasTAIHelper.CAS_REALM_NAME
				+ "'\n\t CAS_VALIDATION_URL='"
				+ CasTAIHelper.CAS_VALIDATION_URL
				+ "'\n\t STORE_PROXY_TICKET='"
				+ CasTAIHelper.STORE_PROXY_TICKET
				+ "'\n\t CAS_CALLBACK_PROXY_URL='"
				+ CasTAIHelper.CAS_CALLBACK_PROXY_URL
				+ "'\n\t CAS_CALLBACK_PROXY_SERVLET='"
				+ CasTAIHelper.CAS_CALLBACK_PROXY_SERVLET
				+ "'\n\t PRINCIPAL_PREFIX='" + CasTAIHelper.PRINCIPAL_PREFIX
				+ "'\n\t PRINCIPAL_SUFFIX='" + CasTAIHelper.PRINCIPAL_SUFFIX
				+ "'\n\t DEBUG='" + CasTAIHelper.DEBUG + "'\n\n"

		);

		if (CasTAIHelper.CAS_VALIDATION_URL == null) {
			System.err
					.println("CAS_VALIDATION_URL property information missing");
			return -1;
		}
		if (CasTAIHelper.PRINCIPAL_PREFIX == null) {
			System.err.println("PRINCIPAL_PREFIX property information missing");
			return -1;
		}
		if (CasTAIHelper.CAS_REALM_NAME == null) {
			System.err.println("CAS_REALM_NAME property information missing");
			return -1;
		}
		if (CasTAIHelper.PRINCIPAL_SUFFIX == null) {
			System.err.println("PRINCIPAL_SUFFIX property information missing");
			return -1;
		}

		if (CasTAIHelper.STORE_PROXY_TICKET == true) {
			if (CasTAIHelper.CAS_CALLBACK_PROXY_URL == null
					|| "".equals(CasTAIHelper.CAS_CALLBACK_PROXY_URL)) {
				System.err
						.println("CAS_CALLBACK_PROXY_URL property information missing, required if STORE_PROXY_TICKET=true");
				return -1;
			}
			if (CasTAIHelper.CAS_CALLBACK_PROXY_SERVLET == null
					|| "".equals(CasTAIHelper.CAS_CALLBACK_PROXY_SERVLET)) {
				System.err
						.println("CAS_CALLBACK_PROXY_SERVLET property information missing, required if STORE_PROXY_TICKET=true");
				return -1;

			}
		}
		return 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.wsspi.security.tai.TrustAssociationInterceptor#isTargetInterceptor(javax.servlet.http.HttpServletRequest)
	 */
	public static boolean isTargetInterceptor(HttpServletRequest request) {
		boolean hasTicket = request.getParameter(PARAM_TICKET) != null;

		if (CasTAIHelper.DEBUG)
			System.out.println("has ticket? =" + hasTicket);
		if (CasTAIHelper.DEBUG)
			System.out.println("request url=" + request.getRequestURL());

		return hasTicket;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.wsspi.security.tai.TrustAssociationInterceptor#negotiateValidateandEstablishTrust(javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse)
	 */
	public static String negotiateValidateandEstablishTrust(String ticket,
			HttpServletRequest request, ServiceTicketValidator sv)
			throws CasException {

		String user = null;
		String errorCode = null;
		String errorMessage = null;
		String xmlResponse = null;

		if (CasTAIHelper.DEBUG)
			System.out.println("new ticket validator");

		String callBackUrl = CasTAIHelper.CAS_CALLBACK_PROXY_URL
				+ request.getContextPath() + CAS_CALLBACK_PROXY_SERVLET;
		String service = getServiceURL(request);

		sv.setCasValidateUrl(CasTAIHelper.CAS_VALIDATION_URL);

		if (CasTAIHelper.DEBUG)
			System.out.println("set validation url :"
					+ CasTAIHelper.CAS_VALIDATION_URL);

		if (CasTAIHelper.STORE_PROXY_TICKET) {
			if (CasTAIHelper.DEBUG)
				System.out.println("set callback url :" + callBackUrl);

			sv.setProxyCallbackUrl(callBackUrl);
		}

		if (CasTAIHelper.DEBUG)
			System.out.println("set service : " + service);
		sv.setServiceTicket(ticket);

		if (CasTAIHelper.DEBUG)
			System.out.println("set ticket : " + ticket);
		sv.setService(service);

		try {
			sv.validate();
			if (CasTAIHelper.DEBUG)
				System.out.println("validation done");

		} catch (Exception e) {
			if (CasTAIHelper.DEBUG) {
				System.out.println("error during validation : "
						+ e.getMessage());
				e.printStackTrace();
			}
			throw new CasException(e.getMessage());
		}

		xmlResponse = sv.getResponse();
		if (CasTAIHelper.DEBUG)
			System.out.println("got xml response :\n" + xmlResponse);

		if (sv.isAuthenticationSuccesful()) {
			user = sv.getUser();
			if (CasTAIHelper.DEBUG)
				System.out.println("sucess:user=" + user);
		} else {
			errorCode = sv.getErrorCode();
			errorMessage = sv.getErrorMessage();

			if (CasTAIHelper.DEBUG)
				System.out.println("failure:" + "CAS_ERROR_CODE:'" + errorCode
						+ "':" + errorMessage);

			throw new CasException("CAS_ERROR_CODE:'" + errorCode + "':"
					+ errorMessage);
		}

		String uid = CasTAIHelper.PRINCIPAL_PREFIX + user
				+ CasTAIHelper.PRINCIPAL_SUFFIX;
		if (CasTAIHelper.DEBUG)
			System.out.println("cas uid : " + uid);
		return uid;

	}

	public static String getTicket(HttpServletRequest request)
			throws ServiceTicketLostException {
		if (CasTAIHelper.DEBUG)
			System.out.println(" try to get ticket");
		String ticket = request.getParameter("ticket");
		if (ticket != null) {
			if (CasTAIHelper.DEBUG)
				System.out.println("ticket not null : " + ticket);
			return ticket;
		} else {
			System.err
					.println("Ticket disapear between interception and validation, "
							+ "this should never happen");
			throw new ServiceTicketLostException(
					"Ticket disapear between interception and validation, "
							+ "this should never happen");
		}

	}

	/**
	 * borrowed from last java cas client ja sig, removed encode stuff
	 * 
	 * Constructs a service url from the HttpServletRequest or from the given
	 * serviceUrl. Prefers the serviceUrl provided if both a serviceUrl and a
	 * serviceName.
	 * 
	 * @param request
	 *            the HttpServletRequest
	 */
	private static final String getServiceURL(final HttpServletRequest request) {
		final StringBuffer buffer = new StringBuffer();

		buffer.append(request.getRequestURL().toString());
		if (null != request.getQueryString()
				&& !(DEFAULT_PRINCIPAL_PREFIX.equals(request.getQueryString()))) {
			final int location = request.getQueryString().indexOf(
					PARAM_TICKET + "=");

			if (location == 0) {
				final String returnValue = buffer.toString();
				if (DEBUG) {
					System.out.println("serviceUrl generated: " + returnValue);
				}
				return returnValue;
			}

			buffer.append("?");

			if (location == -1) {
				buffer.append(request.getQueryString());
			} else if (location > 0) {
				final int actualLocation = request.getQueryString().indexOf(
						"&" + PARAM_TICKET + "=");

				if (actualLocation == -1) {
					buffer.append(request.getQueryString());
				} else if (actualLocation > 0) {
					buffer.append(request.getQueryString().substring(0,
							actualLocation));
				}
			}
		}

		String returnValue = "";
		returnValue = URLEncoder.encode(buffer.toString());
		if (DEBUG) {
			System.out.println("serviceUrl generated: " + returnValue);
		}
		return returnValue;
	}

}