/*
 * Created on April 11, 2006
 *
 */
package com.octo.cas.client.websphere;

import javax.security.auth.Subject;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.websphere.security.WSSecurityException;
import com.ibm.websphere.security.auth.WSSubject;
import com.octo.cas.client.websphere.CasProxyTicketHolder;

import edu.yale.its.tp.cas.proxy.ProxyTicketReceptor;

/**
 * Utility class that helps to fetch a proxy ticket.
 * @author Marc-Antoine Garrigue
 */
public class ProxyTicketHelper {
  
    private static Log log = LogFactory.getLog(ProxyTicketHelper.class);

    /**
  	 * Extracts CAS ticket(pgt) from Credential
  	 * @return ticket's(pgt) value 
  	 */
  	public static String getPgtIOU() {
  		Subject s;
  		try {
  			s = WSSubject.getCallerSubject();
  			if (log.isDebugEnabled()) {
  				log.debug("subject:" + s + "\npublic credentials: "
  						+ s.getPublicCredentials(CasProxyTicketHolder.class));
  			}
  		} catch (WSSecurityException e) {
  			e.printStackTrace();
  			return null;
  		}
  		if (s != null && s.getPublicCredentials(CasProxyTicketHolder.class) != null) {
  
  			String pgtiou = null;
  			try {
  				pgtiou = ((CasProxyTicketHolder) s.getPublicCredentials(
  						CasProxyTicketHolder.class).iterator().next())
  						.getPgtIou();
  				
  				if (log.isDebugEnabled()) {
  					log.debug("pgtiou : " + pgtiou);
  				}
  			} catch (RuntimeException e) {
  				log.error("Error while extracting CAS ticket",e);
  			}
  			return pgtiou;
  		}
  		if (log.isDebugEnabled()) {
  			log.debug("no pgtiou found");
  		}
  		return null;
  	}
  	
  	/**
  	 * Get proxyticket from a pgtIOu
  	 * @return proxy ticket
  	 */
  	public static String getProxyTicket(String pgtIOU, String targetUrl) {
  	  String proxyTicket = null;
				try {
  	      proxyTicket = ProxyTicketReceptor.getProxyTicket(pgtIOU, targetUrl);
  	      if (log.isDebugEnabled()) {
						log.debug("get pt:" + proxyTicket);
					}
				}  catch (Throwable e) {
					log.error("Error while receiving pt", e);
				}
				return proxyTicket;
  	}


}